# Generated by Django 4.1.7 on 2023-03-25 06:49

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('contenttypes', '0002_remove_content_type_name'),
    ]

    operations = [
        migrations.CreateModel(
            name='React',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.CharField(max_length=20, unique=True)),
                ('emoji', models.CharField(max_length=5)),
                ('source', models.ImageField(blank=True, null=True, upload_to='react_source')),
            ],
        ),
        migrations.CreateModel(
            name='Reaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('object_id', models.PositiveIntegerField()),
                ('urlhash', models.CharField(editable=False, max_length=50, unique=True)),
                ('content_type', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='contenttypes.contenttype')),
            ],
        ),
        migrations.CreateModel(
            name='UserReaction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('react', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to='reaction.react')),
                ('reaction', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to='reaction.reaction')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='reactions', to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ReactionSettings',
            fields=[
                ('name', models.CharField(max_length=30)),
                ('slug', models.SlugField(help_text='This value will be used in render_reaction tag', primary_key=True, serialize=False)),
                ('react_type', models.CharField(choices=[('e', 'emoji'), ('s', 'source')], default='e', max_length=1)),
                ('react_emoji', models.ManyToManyField(related_name='react_emojis', to='reaction.react')),
            ],
            options={
                'verbose_name': 'Reaction Settings',
                'verbose_name_plural': 'Reaction Settings',
            },
        ),
        migrations.AddField(
            model_name='reaction',
            name='settings',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, to='reaction.reactionsettings'),
        ),
    ]



from django.contrib import admin

from reaction.models import React, Reaction, UserReaction, ReactionSettings


class ReactAdmin(admin.ModelAdmin):
    list_display = ('slug', 'emoji', 'source_file')
    search_fields = ('slug', 'emoji')


class ReactionAdmin(admin.ModelAdmin):
    list_display = ('content_obj', 'urlhash', 'settings')
    readonly_fields = ('content_type', 'urlhash', 'object_id', 'content_object', 'settings')

    def content_obj(self, obj):
        return f'{obj.content_type} - {obj.content_object}'

    content_obj.short_description = 'Content Object'


class UserReactionAdmin(admin.ModelAdmin):
    list_display = ('user', 'react', 'reaction')
    readonly_fields = ('user', 'react', 'reaction')


class SettingsAdmin(admin.ModelAdmin):
    list_display = ('name', 'slug', 'react_type')
    search_fields = ('name', 'slug')
    prepopulated_fields = {'slug': ('name',)}


admin.site.register(React, ReactAdmin)
admin.site.register(Reaction, ReactionAdmin)
admin.site.register(UserReaction, UserReactionAdmin)
admin.site.register(ReactionSettings, SettingsAdmin)



from django.conf import settings

_REACTION_SETTINGS = getattr(settings, 'REACTION_SETTINGS', {})

REACTION_SETTINGS = {
    'URLHASH_LENGTH': _REACTION_SETTINGS.get('URLHASH_LENGTH', 8),
    'OFFLINE_IMPORTS': _REACTION_SETTINGS.get('OFFLINE_IMPORTS', True),
}



from random import choice
from string import ascii_lowercase

from django.contrib.auth import get_user_model
from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.utils.html import format_html

from reaction import settings

User = get_user_model()


class React(models.Model):
    slug = models.CharField(max_length=20, unique=True)
    emoji = models.CharField(max_length=5)
    source = models.ImageField(upload_to='react_source', null=True, blank=True)

    def __str__(self):
        return f'{self.emoji} ({self.slug})'

    def source_file(self):
        return format_html(f"<img style='height:20px' src='{self.source.url}' alt='{self.emoji}'>") if self.source else 'X'


class ReactionSettings(models.Model):
    name = models.CharField(max_length=30)
    slug = models.SlugField(primary_key=True, help_text='This value will be used in render_reaction tag')
    react_type = models.CharField(max_length=1, choices=(('e', 'emoji'), ('s', 'source')), default='e')
    react_emoji = models.ManyToManyField(React, related_name='react_emojis')

    class Meta:
        verbose_name = 'Reaction Settings'
        verbose_name_plural = 'Reaction Settings'

    def __str__(self):
        return f'{self.name} - [{self.slug}]'


class ReactionQuerySet(models.QuerySet):
    def get_reacts(self):
        reacts = React.objects.all()
        return {react: self.filter(react=react) for react in reacts}


class Reaction(models.Model):
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    object_id = models.PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')

    settings = models.ForeignKey(ReactionSettings, null=True, on_delete=models.SET_NULL)
    urlhash = models.CharField(max_length=50, unique=True, editable=False)

    objects = ReactionQuerySet.as_manager()

    def __str__(self):
        return f'{self.content_type} - {self.content_object}'

    @staticmethod
    def generate_urlhash():
        return ''.join(choice(ascii_lowercase) for _ in range(settings.URLHASH_LENGTH))

    def set_unique_urlhash(self):
        if not self.urlhash:
            self.urlhash = self.generate_urlhash()
            while self.__class__.objects.filter(urlhash=self.urlhash).exists():
                self.urlhash = self.generate_urlhash()

    def save(self, *args, **kwargs):
        self.set_unique_urlhash()
        super(Reaction, self).save(*args, **kwargs)


class UserReaction(models.Model):
    user = models.ForeignKey(User, on_delete=models.CASCADE, related_name='reactions')
    react = models.ForeignKey(React, related_name='reactions', on_delete=models.CASCADE)
    reaction = models.ForeignKey(Reaction, on_delete=models.CASCADE, related_name='reactions')
    created = models.DateTimeField(auto_now_add=True)
    updated = models.DateTimeField(auto_now=True)

    def __str__(self):
        return f'{self.user} - {self.react.emoji}'

    def save(self, *args, **kwargs):
        allowed_reacts = [react.id for react in Reaction.objects.get(id=self.reaction.id).settings.react_emoji.all()]
        if self.react.id in allowed_reacts:
            super(UserReaction, self).save(*args, **kwargs)
        else:
            raise PermissionError('React not allowed!')



from django.db import migrations


def InitialReact(apps, schema_editor):
    React = apps.get_model('reaction', 'React')
    React.objects.create(slug='like', emoji='👍')
    React.objects.create(slug='dislike', emoji='👎')
    React.objects.create(slug='heart', emoji='❤️')


def InitialReactionSettings(apps, schema_editor):
    ReactionSettings = apps.get_model('reaction', 'ReactionSettings')
    reaction_settings = ReactionSettings.objects.create(name='Default Config', slug='default-config', react_type='e')
    # Set some emojis to settings
    reacts = apps.get_model('reaction', 'React').objects.filter(slug__in=['like', 'dislike'])
    reaction_settings.react_emoji.set(reacts)


class Migration(migrations.Migration):
    dependencies = [
        ('reaction', '0001_initial'),
    ]

    operations = [
        migrations.RunPython(InitialReact),
        migrations.RunPython(InitialReactionSettings),
    ]



from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import HttpResponse
from django.shortcuts import render
from django.views import View

from reaction.models import React, Reaction, UserReaction


class ReactionView(View, LoginRequiredMixin):
    def get(self, request, *args, **kwargs):
        urlhash = request.GET.get('urlhash')
        reaction = Reaction.objects.get(urlhash=urlhash)
        user_react = UserReaction.objects.get(user=request.user, reaction__urlhash=urlhash).react if request.user.is_authenticated and UserReaction.objects.filter(user=request.user, reaction__urlhash=urlhash).exists() else None
        reacts = {react: reaction.reactions.filter(react=react).count() for react in reaction.settings.react_emoji.all()}

        context = {
            'reaction': reaction,
            'user_react': user_react,
            'reacts': reacts
        }
        return render(request, 'reaction/reaction/reaction.html', context=context)

    def post(self, request, *args, **kwargs):
        user = request.user
        reaction_urlhash = request.POST.get('urlhash', None)
        react_slug = request.POST.get('react_slug', None)

        reaction = UserReaction.objects.filter(user=user, reaction__urlhash=reaction_urlhash).first()

        if reaction:  # Update Previous Reaction
            if reaction.react.slug == react_slug:  # Delete Previous Reaction
                reaction.delete()
            else:  # Change Previous Reaction
                react = React.objects.get(slug=react_slug)
                reaction.react = react
                reaction.save()
        else:  # Create New Reaction
            reaction = Reaction.objects.get(urlhash=reaction_urlhash)
            react = React.objects.get(slug=react_slug)
            UserReaction.objects.create(user=user, reaction=reaction, react=react)

        return HttpResponse(status=200)



from django import template
from django.contrib.contenttypes.models import ContentType

from reaction import settings
from reaction.models import Reaction, ReactionSettings

register = template.Library()


@register.inclusion_tag('reaction/reaction/reaction_form.html')
def render_reaction(request, obj, settings_slug):
    app_name = type(obj)._meta.app_label
    model_name = type(obj).__name__.lower()
    object_id = obj.id
    content_type = ContentType.objects.get(app_label=app_name, model=model_name)
    reaction_settings = ReactionSettings.objects.get(slug=settings_slug)
    reaction = Reaction.objects.get_or_create(content_type=content_type, object_id=object_id, settings=reaction_settings)[0]
    context = {
        'object': obj,
        'request': request,
        'reaction': reaction,
    }
    return context


@register.filter
def number(value, floating_points=None):
    converters = {
        3: 'K',
        6: 'M',
        9: 'B',
    }

    if floating_points is not None and (10 ** 12 > value >= 1000):
        for exponent, converter in converters.items():
            large_number = 10 ** exponent
            if value >= large_number * 1000:
                continue
            return f'{value / large_number:,.{floating_points}f} {converter}'
    else:
        return f'{value:,}'


@register.inclusion_tag('reaction/utils/IMPORTS.html')
def render_reaction_import():
    return {'offline_imports': settings.OFFLINE_IMPORTS}


@register.inclusion_tag('reaction/utils/SCRIPTS.html')
def render_reaction_script():
    pass
